<?php
/*
Plugin Name: Hammy
Plugin URI: http://wordpress.org/extend/plugins/hammy/
Description: Creates responsive images for your content area with breakpoints that you set.
Author: Noel Tock
Version: 0.1
Author URI: http://www.noeltock.com
*/

/**
 * Defines
 */
define ( 'HAMMY_VERSION', '0.1' );
define ( 'HAMMY_PATH',  WP_PLUGIN_URL . '/' . end( explode( DIRECTORY_SEPARATOR, dirname( __FILE__ ) ) ) );


/**
 * Register Default Settings
 */
if (get_option('hammy_options')== '') {
    register_activation_hook(__FILE__, 'hammy_defaults');
}

function hammy_defaults() {
    $arr = array('hammy_breakpoints' => '320,480,768');
    update_option('hammy_options', $arr);
}


/**
 * Check for WPThumb, include.
 */
if ( !function_exists('wpthumb') ) {
	include_once('WPThumb/wpthumb.php');
}

/**
 * Plugin Options
 */
include_once('includes/options.php');

add_action('admin_init', 'hammy_options_init' );
add_action('admin_menu', 'hammy_add_page');


/**
 * Enqueue & Localize JavaScript
 */
function load_hammy_js() {

	$options = get_option('hammy_options');
	
	wp_enqueue_script('jquery-picture', HAMMY_PATH . '/js/jquery-picture.js', array('jquery') );
	wp_enqueue_script('hammy', HAMMY_PATH . '/js/hammy.js', array('jquery') );
	wp_localize_script( 'hammy', 'imageParent', $options['hammy_parent'] );
		
}

add_action('wp_print_scripts', 'load_hammy_js');


/**
 * Hammy Time
 *
 * @param $content
 * @return DOM element with fallback (<figure> -> <img>)
 */
function hammy_replace_images($content) {

	global $post;
	
	preg_match_all('/<img (.*?)\/>/', $content, $images);
	
	if( !is_null( $images ) ) {
	
		foreach($images[0] as $index => $value) {
		
			$doc = new DOMDocument();
			$doc->loadHTML($value);
			$items = $doc->getElementsByTagName('img');
				
			foreach($items as $item) {
			
				// Get Attributes
			
				$original = $item->getAttribute('src');
				$width = $item->getAttribute('width');
				$class = $item->getAttribute('class');
				$alt = $item->getAttribute('alt');
				
				// Get Sizes
				
				$options = get_option('hammy_options');
				$sizes = explode(",", $options['hammy_breakpoints']);
				
				// Render Sizes
				
				$output = array();
				$i = 0;
				
				foreach ($sizes as $size) {

					if ( $i == 0 ) {
					
						$name = 'data-media';
						
					} else {
					
						$name = 'data-media' . $size;
						
					}
				
					if ( $size < $width ) {
					
						$resized_image = wpthumb( $original, 'width=' . $size . '&crop=0' );
						$output[] = $name . '="' . $resized_image . '"';
					
					}
					
					$i++;
				
				}
												
				$resized_images = implode(" ", $output);
				
				// Output & Replace Strings
				
				$newimage = '<figure class="hammy-responsive ' . $class . '" ' . $resized_images . ' title="' . $alt . '"><noscript><img src="' . $original . '" alt="' . $alt . '"></noscript></figure>';

				$content = str_replace($images[0][$index], $newimage, $content);
				
			}
		
		}
		
	}
	
	return $content;
	
}

add_filter('the_content', 'hammy_replace_images', 999);

?>